// ---------------------------------------------------------------
// Copyright 2006, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Windows.Forms;
using System.Xml;			// for XMLTextWriter and XMLTextReader
using System.IO;			// for StreamReader and StreamWriter
using System.Text;			// for StringBuilder
using SAS.Shared.AddIns;	// for access to the Add-in interfaces

namespace SAS.CustomTasks.AreaBar
{
	/// <summary>
	/// Summary description for AreaBar.
	/// </summary>
	public class AreaBar : ISASTask, ISASTaskDescription, ISASTaskAddIn
	{
		#region private members 
		// consumer object supplied by host application
		private ISASTaskConsumer consumer = null;
		// member variables to store state information
		private string server = string.Empty;
		private string library = string.Empty;
		private string dataset = string.Empty;
		private string category = string.Empty;
		private string response = string.Empty;
		private string width = string.Empty;
		private string wstat = string.Empty;
		private bool   bShowSubgroup;
		private string subgroup = string.Empty;
		private string rstat = string.Empty;

		#endregion

		#region Constructors
		public AreaBar()
		{
		}
		#endregion 

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{
				return true;
			}
		}

		public string AddInName
		{
			get
			{
				return "Area Bar Chart";
			}
		}

		public void Disconnect()
		{
			consumer = null;
		}

		public int Languages(out string[] Items)
		{
			Items = new string[] { "en-US" };

			return 1;
		}

		public string AddInDescription
		{
			get
			{
				return "Area Bar Chart add-in example.";
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			consumer = Consumer;

			return true;
		}
		public string Language
		{
			set
			{
				// we support english only
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				return "BASE";
			}
		}

		public bool StandardCategory
		{
			get
			{
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return "Area Bar Chart";
			}
		}

		public string Validation
		{
			get
			{
				return "SAS Area Bar Chart Sample";
			}
		}

		public string TaskCategory
		{
			get
			{
				return "SAS Custom Task";
			}
		}
		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}
		public string Clsid
		{
			get
			{
				return "67E83B4E-4D3A-42b1-86A0-9E9223A5C92C";
			}
		}
		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}
		public bool RequiresData
		{
			get
			{
				return true;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within our assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.AreaBar.AreaBarGraph.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{
				return "Proc GAreaBar example.";
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				return "Area Bar Chart";
			}
		}

		public string ProcsUsed
		{
			get
			{
				return "PROC GAREABAR;";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				return "Add some stuff here";
			}
		}

		public string ProductsOptional
		{
			get
			{
				return "";
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				return null;
			}
		}

		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}
		public void Terminate()
		{
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = null;
			Label = null;
			return new SAS.Shared.AddIns.OutputData();
		}

		public string SasCode
		{
			get
			{
				StringBuilder sb = new StringBuilder();

				sb.Append("\n%let name=area;\n");

				// GOPTIONS
				sb.Append("goptions border;\n");
				sb.Append("goptions reset=global;\n");
				sb.Append("goptions cback=white;\n");
				sb.Append("goptions hsize=6in vsize=3in;\n");
				sb.Append("goptions colors=(cxffff99 cxccffcc cxccffff cxff99cc cxccccff cx00ccff);\n");

				sb.Append("\ntitle c=black \"Area Bar Chart\";\n");

				sb.AppendFormat("proc gareabar data={0};\n", library + '.' + this.dataset);
				string code = string.Empty;

				if (bShowSubgroup)
					code = "\tvbar {0}*{1} / sumvar={2} cframe=white wstat={3} rstat={4} discrete subgroup={5} des=\"\" name=\"&name\";\n";
				else
					code = "\tvbar {0}*{1} / sumvar={2} cframe=white wstat={3} discrete des=\"\" name=\"&name\";\n";

				sb.AppendFormat(code, category, width, response, wstat, rstat, subgroup);

				sb.Append("run;\n\n");
				sb.Append("quit;\n\n");

				return sb.ToString();
			}
		}

		public bool Initialize()
		{
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// launch UI
			// create the dialog class
			AreaBarForm dlg = new AreaBarForm();

			// set the consumer interface so the dialog can call back into the
			// application
			dlg.Consumer = consumer;
			dlg.CurrentServer = server;
			dlg.Library = library;
			dlg.DataSetName = dataset;
			dlg.CategoryVar = category;
			dlg.ResponseVar = response;
			dlg.WidthVar = width;
			dlg.WSTAT = wstat;
			dlg.ShowSubgroup = bShowSubgroup;
			dlg.SubgroupVar = subgroup;
			dlg.RSTAT = rstat;
			
			if (DialogResult.OK == dlg.ShowDialog())
			{
				server = dlg.CurrentServer;
				library = dlg.Library;
				dataset = dlg.DataSetName;
				category = dlg.CategoryVar;
				response = dlg.ResponseVar;
				width = dlg.WidthVar;
				wstat = dlg.WSTAT;
				bShowSubgroup = dlg.ShowSubgroup;
				subgroup = dlg.SubgroupVar;
				rstat = dlg.RSTAT;

				return SAS.Shared.AddIns.ShowResult.RunNow;
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		public string Label
		{
			get
			{
				return "AreaBarChart";
			}
			set
			{
			}
		}
		public int OutputDataCount
		{
			get
			{		
				return 0;
			}
		}
		#endregion

		#region Save and load XML state 
		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("AreaBarChart");

			writer.WriteElementString("Server", server);

			writer.WriteElementString("Library", library);
			writer.WriteElementString("DataSet", dataset);

			writer.WriteElementString("Category", category);
			writer.WriteElementString("Response", response);			
			writer.WriteElementString("Width", width);

			wstat = (wstat == string.Empty) ? "SUM" : wstat;
			writer.WriteElementString("SpecifyWidthAs", wstat);

			writer.WriteElementString("ShowSubgroup",XmlConvert.ToString(bShowSubgroup));

			writer.WriteElementString("Subgroup", subgroup);

			rstat = (rstat == string.Empty) ? "SUM" : rstat;
			writer.WriteElementString("SpecifySubgroupAs", (rstat == string.Empty) ? "SUM" : rstat);

			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml != null && xml.Length > 0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("AreaBarChart");

					server = reader.ReadElementString("Server");

					library = reader.ReadElementString("Library");
					dataset = reader.ReadElementString("DataSet");
					
					category = reader.ReadElementString("Category");
					response = reader.ReadElementString("Response");
					width = reader.ReadElementString("Width");

					wstat = reader.ReadElementString("SpecifyWidthAs");

					bShowSubgroup = XmlConvert.ToBoolean(reader.ReadElementString("ShowSubgroup"));

					subgroup = reader.ReadElementString("Subgroup");

					rstat = reader.ReadElementString("SpecifySubgroupAs");

					reader.ReadEndElement();

					reader.Close();
				}
				catch
				{
					// Uh oh, what do I do now?
				}
			}
		}
		#endregion
	}
}
