// ---------------------------------------------------------------
// Copyright 2006, SAS Institute Inc.
// ---------------------------------------------------------------
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Runtime.Serialization;
using System.Windows.Forms;

namespace SAS.CustomTasks.AreaBar
{
	public class IconComboBox : System.Windows.Forms.ComboBox
	{
		#region Fields

		private ImageList _ImageList = null;

		private System.ComponentModel.IContainer components = null;

		#endregion // Fields

		#region Constructors

		public IconComboBox()
		{
			// This call is required by the Windows Form Designer.
			InitializeComponent();

			// Set owner draw mode
			this.DrawMode = DrawMode.OwnerDrawFixed;

			// Since we don't currently draw the icons in the edit control of the combobox,
			// force to DropDownList style for now.
			this.DropDownStyle = ComboBoxStyle.DropDownList;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#endregion // Constructors

		#region Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}

		#endregion

		#region Properties

		/// <summary>
		/// Gets or sets the ImageList that contains images that can be displayed in the IconComboBox items.
		/// </summary>
		[DefaultValue(null)]
		public ImageList ImageList
		{
			get { return _ImageList; }
			set { _ImageList = value; }
		}

		#endregion // Properties

		#region Methods

		protected override void OnDrawItem(DrawItemEventArgs e)
		{
			e.DrawBackground();
			e.DrawFocusRectangle();
			IconComboBoxItem item;
			Rectangle bounds = e.Bounds;

			if (e.Index >= 0 && e.Index < Items.Count)
			{
				item = Items[e.Index] as IconComboBoxItem;
				if (item != null)
				{
					if (_ImageList != null && item.ImageIndex > -1 && item.ImageIndex < _ImageList.Images.Count)
					{
						Size imageSize = _ImageList.ImageSize;
						_ImageList.Draw(e.Graphics, bounds.Left, bounds.Top, item.ImageIndex); 
						e.Graphics.DrawString(item.Text, e.Font, new SolidBrush(e.ForeColor), 
							bounds.Left + imageSize.Width, bounds.Top);
					}
					else
					{
						e.Graphics.DrawString(item.Text, e.Font, new SolidBrush(e.ForeColor),
							bounds.Left, bounds.Top);
					}
				}
				else
				{
					e.Graphics.DrawString(Items[e.Index].ToString(), e.Font, 
						new SolidBrush(e.ForeColor), bounds.Left, bounds.Top);
				}
			}
			else
			{
				e.Graphics.DrawString(Text, e.Font, new SolidBrush(e.ForeColor),
					bounds.Left, bounds.Top);
			}

			base.OnDrawItem(e);
		}


		#endregion // Methods
	}

	public class IconComboBoxItem : Component
	{
		#region Fields

		private string _Text = string.Empty;
		private int _ImageIndex = -1;
		private object _Tag = null;
		internal IconComboBox _IconComboBox = null;

		#endregion // Fields
				
		#region Constructors

		public IconComboBoxItem()
		{}
		public IconComboBoxItem(string text, int imageIndex)
		{
			_Text = text;
			_ImageIndex = imageIndex;
		}

		public IconComboBoxItem(string text): this(text, -1)
		{}

		#endregion // Constructors

		#region Properties

		/// <summary>
		/// Gets or sets the text to be displayed in the IconComboBoxItem.
		/// </summary>
		[Localizable(true)]
		public string Text
		{
			get { return _Text; }
			set { _Text = value; }
		}
		
		/// <summary>
		/// Gets or sets the index of the image (in the IconComboBox's Images ImageList) to be displayed in the IconComboBoxItem.
		/// </summary>
		[Localizable(true),
		DefaultValue(-1),
		TypeConverter(typeof(ImageIndexConverter)),
		DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden),
		Editor("System.Windows.Forms.Design.ImageIndexEditor, System.Design, Version=1.0.5000.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", typeof(UITypeEditor))]
		public int ImageIndex
		{
			get
			{
				if (_ImageIndex != -1 && this.ImageList != null && _ImageIndex >= this.ImageList.Images.Count)
					return this.ImageList.Images.Count - 1;

				return _ImageIndex;
			}

			set
			{
				if (value < -1)
					throw new ArgumentException("ImageIndex cannot be less than -1.");

				_ImageIndex = value;
			}
		}

		/// <summary>
		/// Gets or sets an object that contains data to associate with the item.
		/// </summary>
		[DefaultValue(null)]
		public object Tag
		{
			get { return _Tag; }
			set { _Tag = value; }
		}

		[Browsable(false)]
		public ImageList ImageList
		{
			get
			{
				if (_IconComboBox != null)
					return _IconComboBox.ImageList;

				return null;
			}
		}

		#endregion // Properties

		#region Methods

		public override string ToString()
		{
			return _Text;
		}

		#endregion // Methods
	}

}
