using System;
using System.Windows.Forms;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder
using System.Data;
using System.Data.OleDb;

// SAS AddIns namespace
using SAS.Shared.AddIns;

namespace SAS.CustomTasks.FSEdit
{
	/// <summary>
	/// A custom task for use in SAS Enterprise Guide or the SAS Add-In for Microsoft Office
	/// </summary>
	public class FSEdit : ISASTask, ISASTaskAddIn, ISASTaskDescription
	{
		// change these values for your task
		#region private members for boilerplate values
		
		// replace these boilerplate values with values that apply to your task
		private string sLabel = "Observation Editor";
		private string sAddInName = "Observation Editor";
		private string sAddInDescription = "Edit one data observation at a time.";
		private string sProductsReq = "BASE";
		private string sProductsOpt = "";
		private string sTaskName= "Observation Editor";
		private string sCategory = "SAS Custom Task";
		private string sTaskDescription = "Edit one data observation at a time";
		private string sFriendlyName = "Observation Editor";
		private string sWhatIsDescription = "Edit one data observation at a time";
		
		#endregion
		
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;

		public FSEdit()
		{
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;
	
			// this is a good time to get the consumer.ActiveData, if your task requires it.

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				return false;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				// PLACEHOLDER: replace with your own category
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID generated by the template
				return "293D7B33-6F1B-4B3C-AABE-69EDF9165F86";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				return true;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.FSEdit.FSEditIcon.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				return false;
			}
		}

		public string FriendlyName
		{
			get
			{
				// Replace with a user-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}

		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// Cleanup as needed
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			// no output data created
			Source = null;
			Label = null;
			return SAS.Shared.AddIns.OutputData.Unknown;

		}

		public string SasCode
		{
			get
			{
				return "";
			}
		}

		public bool Initialize()
		{
			// initialize this instance of your task
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			string server = "";
			SAS.Workspace ws = null;
			try
			{
				server = consumer.AssignedServer;
				ws = consumer.Workspace(server) as SAS.Workspace;
			}
			catch (Exception ex)
			{
				throw new System.Exception("ISASTaskConsumer.Workspace is not usable!",ex);
			}

			// Show the default form for this custom task
			if (server.Length>0 && ws!=null)
			{
				FSEditForm dlg = new FSEditForm(consumer, ws.UniqueIdentifier);
				dlg.Text = sLabel;

				if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
					return SAS.Shared.AddIns.ShowResult.Canceled;
				else
					return SAS.Shared.AddIns.ShowResult.Canceled;
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}

		public int OutputDataCount
		{
			get
			{				
				// does this task create any output data sets?
				return 0;
			}
		}

		#endregion

		// Augment these methods to save and retrieve the state of your task
		#region Helper methods for serialization 

		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("FSEdit");
			// add more WriteElement calls here to save the state of your task
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("FSEdit");
					// add more ReadElement calls here to read in the state of your task
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}

		#endregion
	}
}
