using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.OleDb;
using SAS.Shared.AddIns;

namespace SAS.CustomTasks.FSEdit
{
	/// <summary>
	/// Summary description for FSEditForm.
	/// </summary>
	public class FSEditForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button btnOK;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.NumericUpDown udObs;
		private System.Windows.Forms.Button bRight;
		private System.Windows.Forms.Button bLeft;
		private System.Windows.Forms.Button bCommit;
		private System.Windows.Forms.Button bDelete;
		private System.Windows.Forms.TabControl tabControl1;
		private System.Windows.Forms.TabPage tabPage1;
		private System.Windows.Forms.TabPage tabPage2;
		private System.Windows.Forms.Button bInsert;
		private System.Windows.Forms.PropertyGrid insertPropertyGrid;
		private System.Windows.Forms.PropertyGrid editPropertyGrid;
		private System.Data.DataTable dt;
		private System.Data.DataSet ds;
		private OleDbDataAdapter adapter;
		private OleDbConnection dbConnection;
		private String strLibrary;
		private String strMember;
		private String strUniqueIdentifier;
		private String strSelectClause;
		private int nRow;
		private bool bEmptyDataSource;
		private ISASTaskConsumer consumer = null;
		private CustomPropertyTable editTable;
		private CustomPropertyTable insertTable;
		private bool bSupressUpdates = false;

		// the ISASTaskConsumer from the host application
		public ISASTaskConsumer Consumer
		{
			set
			{
				consumer = value;
			}
		}
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		#region Constructors/Dispose
		public FSEditForm(ISASTaskConsumer con, String strUI)
		{
			InitializeComponent();
			consumer = con;
			strLibrary = consumer.ActiveData.Library;
			strMember = consumer.ActiveData.Member;
			strSelectClause = "select * from " + strLibrary + "." + strMember;
			strUniqueIdentifier = strUI;
			dt = new DataTable(strMember);
			ds = new System.Data.DataSet();
			ds.Tables.Add(dt);
			nRow = (int)udObs.Value;
			editTable = new CustomPropertyTable();
			insertTable = new CustomPropertyTable();

			if (strUniqueIdentifier.Length>0)
			{
				dbConnection = new OleDbConnection( "Provider=sas.iomprovider.1; SAS Workspace ID=" + strUniqueIdentifier );
				adapter = new OleDbDataAdapter(strSelectClause, dbConnection);
				try
				{
					dbConnection.Open();

					//Count the number of rows in the source
					DataTable temp = new DataTable();
					adapter.Fill(temp);
					if (temp.Rows.Count <= 0)
					{
						udObs.Maximum = 1;
						udObs.Value = 1;
						bEmptyDataSource = true;
					}
					else
					{
						udObs.Maximum = temp.Rows.Count; 
						bEmptyDataSource = false;
					}

					//Fill in the dataset to display the current row number
					//Note: making nRow 0 based
					adapter.Fill(ds, nRow-1, 1, strMember);
					PopulatePropertyGrid(editPropertyGrid, editTable, true);
					PopulatePropertyGrid(insertPropertyGrid, insertTable, false);
				}
				catch
				{
				}
			}
		}

		

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			try
			{
				dbConnection.Close();
				adapter.Dispose();
			}
			catch
			{
			}
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(FSEditForm));
			this.btnOK = new System.Windows.Forms.Button();
			this.label1 = new System.Windows.Forms.Label();
			this.udObs = new System.Windows.Forms.NumericUpDown();
			this.bRight = new System.Windows.Forms.Button();
			this.bLeft = new System.Windows.Forms.Button();
			this.bCommit = new System.Windows.Forms.Button();
			this.editPropertyGrid = new System.Windows.Forms.PropertyGrid();
			this.bDelete = new System.Windows.Forms.Button();
			this.tabControl1 = new System.Windows.Forms.TabControl();
			this.tabPage1 = new System.Windows.Forms.TabPage();
			this.tabPage2 = new System.Windows.Forms.TabPage();
			this.bInsert = new System.Windows.Forms.Button();
			this.insertPropertyGrid = new System.Windows.Forms.PropertyGrid();
			((System.ComponentModel.ISupportInitialize)(this.udObs)).BeginInit();
			this.tabControl1.SuspendLayout();
			this.tabPage1.SuspendLayout();
			this.tabPage2.SuspendLayout();
			this.SuspendLayout();
			// 
			// btnOK
			// 
			this.btnOK.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Right)));
			this.btnOK.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.btnOK.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnOK.Location = new System.Drawing.Point(224, 192);
			this.btnOK.Name = "btnOK";
			this.btnOK.Size = new System.Drawing.Size(80, 23);
			this.btnOK.TabIndex = 7;
			this.btnOK.Text = "OK";
			this.btnOK.Click += new System.EventHandler(this.btnOK_Click);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(8, 8);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(80, 23);
			this.label1.TabIndex = 0;
			this.label1.Text = "Current Obs:";
			this.label1.TextAlign = System.Drawing.ContentAlignment.MiddleCenter;
			// 
			// udObs
			// 
			this.udObs.Font = new System.Drawing.Font("Microsoft Sans Serif", 11.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.udObs.Location = new System.Drawing.Point(120, 8);
			this.udObs.Minimum = new System.Decimal(new int[] {
																  1,
																  0,
																  0,
																  0});
			this.udObs.Name = "udObs";
			this.udObs.Size = new System.Drawing.Size(48, 24);
			this.udObs.TabIndex = 2;
			this.udObs.Value = new System.Decimal(new int[] {
																1,
																0,
																0,
																0});
			this.udObs.ValueChanged += new System.EventHandler(this.udObs_ValueChanged);
			// 
			// bRight
			// 
			this.bRight.Image = ((System.Drawing.Image)(resources.GetObject("bRight.Image")));
			this.bRight.Location = new System.Drawing.Point(144, 8);
			this.bRight.Name = "bRight";
			this.bRight.Size = new System.Drawing.Size(32, 24);
			this.bRight.TabIndex = 3;
			this.bRight.Click += new System.EventHandler(this.bRight_Click);
			// 
			// bLeft
			// 
			this.bLeft.Image = ((System.Drawing.Image)(resources.GetObject("bLeft.Image")));
			this.bLeft.Location = new System.Drawing.Point(88, 8);
			this.bLeft.Name = "bLeft";
			this.bLeft.Size = new System.Drawing.Size(32, 23);
			this.bLeft.TabIndex = 1;
			this.bLeft.Click += new System.EventHandler(this.bLeft_Click);
			// 
			// bCommit
			// 
			this.bCommit.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.bCommit.Location = new System.Drawing.Point(112, 192);
			this.bCommit.Name = "bCommit";
			this.bCommit.Size = new System.Drawing.Size(104, 23);
			this.bCommit.TabIndex = 6;
			this.bCommit.Text = "Commit Changes";
			this.bCommit.Click += new System.EventHandler(this.bCommit_Click);
			// 
			// editPropertyGrid
			// 
			this.editPropertyGrid.CommandsVisibleIfAvailable = true;
			this.editPropertyGrid.HelpVisible = false;
			this.editPropertyGrid.LargeButtons = false;
			this.editPropertyGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.editPropertyGrid.Location = new System.Drawing.Point(8, 40);
			this.editPropertyGrid.Name = "editPropertyGrid";
			this.editPropertyGrid.PropertySort = System.Windows.Forms.PropertySort.NoSort;
			this.editPropertyGrid.Size = new System.Drawing.Size(296, 144);
			this.editPropertyGrid.TabIndex = 4;
			this.editPropertyGrid.Text = "PropertyGrid";
			this.editPropertyGrid.ToolbarVisible = false;
			this.editPropertyGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.editPropertyGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// bDelete
			// 
			this.bDelete.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.bDelete.Location = new System.Drawing.Point(8, 192);
			this.bDelete.Name = "bDelete";
			this.bDelete.Size = new System.Drawing.Size(96, 23);
			this.bDelete.TabIndex = 5;
			this.bDelete.Text = "Delete Obs";
			this.bDelete.Click += new System.EventHandler(this.bDelete_Click);
			// 
			// tabControl1
			// 
			this.tabControl1.Controls.Add(this.tabPage1);
			this.tabControl1.Controls.Add(this.tabPage2);
			this.tabControl1.Location = new System.Drawing.Point(8, 8);
			this.tabControl1.Name = "tabControl1";
			this.tabControl1.SelectedIndex = 0;
			this.tabControl1.Size = new System.Drawing.Size(320, 248);
			this.tabControl1.TabIndex = 0;
			// 
			// tabPage1
			// 
			this.tabPage1.Controls.Add(this.bRight);
			this.tabPage1.Controls.Add(this.btnOK);
			this.tabPage1.Controls.Add(this.udObs);
			this.tabPage1.Controls.Add(this.bLeft);
			this.tabPage1.Controls.Add(this.bDelete);
			this.tabPage1.Controls.Add(this.bCommit);
			this.tabPage1.Controls.Add(this.editPropertyGrid);
			this.tabPage1.Controls.Add(this.label1);
			this.tabPage1.Location = new System.Drawing.Point(4, 22);
			this.tabPage1.Name = "tabPage1";
			this.tabPage1.Size = new System.Drawing.Size(312, 222);
			this.tabPage1.TabIndex = 0;
			this.tabPage1.Text = "Edit Existing Obs";
			// 
			// tabPage2
			// 
			this.tabPage2.Controls.Add(this.bInsert);
			this.tabPage2.Controls.Add(this.insertPropertyGrid);
			this.tabPage2.Location = new System.Drawing.Point(4, 22);
			this.tabPage2.Name = "tabPage2";
			this.tabPage2.Size = new System.Drawing.Size(312, 222);
			this.tabPage2.TabIndex = 1;
			this.tabPage2.Text = "Insert New Obs";
			// 
			// bInsert
			// 
			this.bInsert.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.bInsert.Location = new System.Drawing.Point(200, 192);
			this.bInsert.Name = "bInsert";
			this.bInsert.Size = new System.Drawing.Size(104, 23);
			this.bInsert.TabIndex = 1;
			this.bInsert.Text = "Insert New Obs";
			this.bInsert.Click += new System.EventHandler(this.bInsert_Click);
			// 
			// insertPropertyGrid
			// 
			this.insertPropertyGrid.CommandsVisibleIfAvailable = true;
			this.insertPropertyGrid.HelpVisible = false;
			this.insertPropertyGrid.LargeButtons = false;
			this.insertPropertyGrid.LineColor = System.Drawing.SystemColors.ScrollBar;
			this.insertPropertyGrid.Location = new System.Drawing.Point(8, 8);
			this.insertPropertyGrid.Name = "insertPropertyGrid";
			this.insertPropertyGrid.PropertySort = System.Windows.Forms.PropertySort.NoSort;
			this.insertPropertyGrid.Size = new System.Drawing.Size(296, 176);
			this.insertPropertyGrid.TabIndex = 0;
			this.insertPropertyGrid.Text = "propertyGrid2";
			this.insertPropertyGrid.ToolbarVisible = false;
			this.insertPropertyGrid.ViewBackColor = System.Drawing.SystemColors.Window;
			this.insertPropertyGrid.ViewForeColor = System.Drawing.SystemColors.WindowText;
			// 
			// FSEditForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(336, 261);
			this.Controls.Add(this.tabControl1);
			this.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.Name = "FSEditForm";
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "SAS Custom Task";
			((System.ComponentModel.ISupportInitialize)(this.udObs)).EndInit();
			this.tabControl1.ResumeLayout(false);
			this.tabPage1.ResumeLayout(false);
			this.tabPage2.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

		#region Utility Functions

		private bool ValidateTableContents(CustomPropertyTable table)
		{
			DataColumn dataColumn;
			for (int i = 0; i < ds.Tables[strMember].Columns.Count; i++)
			{
				dataColumn = ds.Tables[strMember].Columns[i];
				if (table[dataColumn.ColumnName].ToString().Equals(String.Empty))
					return false;
			}
			return true;
		}

		private bool HasTableChanged(CustomPropertyTable table)
		{
			//The check for nRow is to prevent updates being possible when an insertion has happened
			if (!bEmptyDataSource && nRow == (int)udObs.Value)
			{
				DataColumn dataColumn;
				for (int i = 0; i < ds.Tables[strMember].Columns.Count; i++)
				{
					dataColumn = ds.Tables[strMember].Columns[i];
					if (!table[dataColumn.ColumnName].ToString().Equals(ds.Tables[strMember].Rows[0][i].ToString()))
						return true;
				}
			}
			return false;
		}

		private void PopulatePropertyGrid(PropertyGrid grid, CustomPropertyTable table, bool bIncludeValues)
		{
			DataColumn dataColumn;
			for (int i = 0; i < ds.Tables[strMember].Columns.Count; i++)
			{
				dataColumn = ds.Tables[strMember].Columns[i];
				table.CustomProperties.Add(new CustomProperty(dataColumn.ColumnName, dataColumn.DataType.ToString(), null));
				if (bIncludeValues && !bEmptyDataSource)
					table[dataColumn.ColumnName] = ds.Tables[strMember].Rows[0][i];
				else
					table[dataColumn.ColumnName] = "";
			}
			grid.SelectedObject = table;
		}

		private void CopyTable(CustomPropertyTable source, CustomPropertyTable target)
		{
			DataColumn dataColumn;
			for (int i = 0; i < ds.Tables[strMember].Columns.Count; i++)
			{
				dataColumn = ds.Tables[strMember].Columns[i];
				target.CustomProperties.Add(new CustomProperty(dataColumn.ColumnName, dataColumn.DataType.ToString(), null));
				target[dataColumn.ColumnName] = source[dataColumn.ColumnName];
			}
		}

		#endregion

		#region Data Functions

		private void UpdateData(String strCode)
		{
			Cursor c = Cursor.Current;
			Cursor.Current = Cursors.WaitCursor;
			if (strCode.Length > 0)
				consumer.Submit.SubmitCode(strCode, consumer.AssignedServer, false);

			Cursor.Current = c;
		}

		private String BuildUpdateSqlCode()
		{
			DataColumn dataColumn;
			String strCode, strQuote = "";
			DataTable table = ds.Tables[strMember];
			int nColumnCount = table.Columns.Count;

			strCode = "Proc SQL UNDO_POLICY=OPTIONAL;";

			for (int colIdx = 0; colIdx < nColumnCount; colIdx++)
			{
				//Create the set statement for the value that is to be changed
				dataColumn = table.Columns[colIdx];
				strQuote = ((dataColumn.DataType.ToString().Equals("System.String")) ? "'":"");
				strCode += " Update ";
				strCode += ( strLibrary + "." + strMember );
				strCode += " Set " + dataColumn.ColumnName  + " = " + strQuote + editTable[dataColumn.ColumnName].ToString() + strQuote;
				strCode += " where "; 

				//create the where clause to encompass all the values of this row to ensure uniqueness.  
				//NOTE: If there are duplicate rows in the datasource that are identical, they will be updated as well because they match the same SQL where statement.  I am not sure if there is a way to prevent this.
				for (int columnIdx = 0; columnIdx < nColumnCount; columnIdx++)
				{
					DataColumn dc = table.Columns[columnIdx];
					strQuote = ((dc.DataType.ToString().Equals("System.String")) ? "'":"");
					strCode += dc.ColumnName + " = " + strQuote + table.Rows[0][columnIdx].ToString() + strQuote;
					if (columnIdx + 1 < nColumnCount)
						strCode += " and ";
				}

				//change the value in the original array to keep it in sync with the change from the foregoing SQL update statement
				//this way if more than one value in the same row has been changed, the update statement remains correct because each subsequent where clause has the newer values
				table.Rows[0].BeginEdit();
				table.Rows[0][colIdx] = editTable[dataColumn.ColumnName].ToString();
				table.Rows[0].EndEdit();
				strCode += ";";
			}
			
			strCode += " QUIT;";

			return strCode;
		}

		private String BuildInsertSqlCode()
		{
			DataColumn dataColumn;
			String strCode, strQuote = "";
			DataTable table = ds.Tables[strMember];
			int nColumnCount = table.Columns.Count;

			strCode = "Proc SQL UNDO_POLICY=OPTIONAL;";
			strCode += " Insert into ";
			strCode += ( strLibrary + "." + strMember + " values(" );
			
			for (int colIdx = 0; colIdx < nColumnCount; colIdx++)
			{
				//Create the set statement for the value that is to be changed
				dataColumn = table.Columns[colIdx];
				strQuote = ((dataColumn.DataType.ToString().Equals("System.String")) ? "'":"");
				strCode += strQuote + insertTable[dataColumn.ColumnName].ToString() + strQuote;
				if ((colIdx + 1) < nColumnCount)
					strCode += ", ";
			}

			strCode += ");";
			strCode += " QUIT;";

			return strCode;
		}

		private String BuildDeleteSqlCode()
		{
			DataColumn dataColumn;
			String strCode, strQuote = "";
			DataTable table = ds.Tables[strMember];
			int nColumnCount = table.Columns.Count;

			strCode = "Proc SQL UNDO_POLICY=OPTIONAL;";
			strCode += " delete from " + ( strLibrary + "." + strMember ) + " where ";

			for (int colIdx = 0; colIdx < nColumnCount; colIdx++)
			{
				dataColumn = table.Columns[colIdx];
				strQuote = ((dataColumn.DataType.ToString().Equals("System.String")) ? "'":"");
				strCode += dataColumn.ColumnName + " = " + strQuote + table.Rows[0][colIdx].ToString() + strQuote;
				if (colIdx + 1 < nColumnCount)
					strCode += " and ";
			}

			strCode += ";";
			strCode += " QUIT;";

			return strCode;
		}

		private void UpdateGrid()
		{
			if (!bEmptyDataSource)
			{
				bCommit.Enabled = true;
				nRow = (int)udObs.Value;
				ds.Tables[strMember].Clear();
				//Note: making nRow 0 based
				adapter.Fill(ds, nRow-1, 1, strMember);
				editTable.CustomProperties.Clear();
				PopulatePropertyGrid(editPropertyGrid, editTable, true);
			}
			else
				bCommit.Enabled = false;
		}
		#endregion

		#region UI EventHandlers

		private void btnOK_Click(object sender, System.EventArgs e)
		{
			//In the case where the user updates the grid, but clicks on Ok, we need to submit the code and update the dataset
			//if any changes have been made to the current row, generate the SQL code and submit it
			if (HasTableChanged(editTable))
			{
				String strCode = BuildUpdateSqlCode();
				UpdateData(strCode);
			}
		}

		private void bRight_Click(object sender, System.EventArgs e)
		{
			if ((udObs.Value + 1) <= udObs.Maximum)
				udObs.Value++;
		}

		private void bLeft_Click(object sender, System.EventArgs e)
		{
			if ((udObs.Value - 1) >= udObs.Minimum)
				udObs.Value--;
		}

		private void bCommit_Click(object sender, System.EventArgs e)
		{
			//if any changes have been made to the current row, generate the SQL code and submit it
			if (HasTableChanged(editTable))
			{
				String strCode = BuildUpdateSqlCode();
				UpdateData(strCode);
			}
		}

		private void bDelete_Click(object sender, System.EventArgs e)
		{
			String strCode = BuildDeleteSqlCode();
			UpdateData(strCode);
			if (udObs.Maximum - 1 >= udObs.Minimum)
				udObs.Maximum--;
			else
			{
				bEmptyDataSource = true;  //we just deleted that last obs of the data source
				editTable.CustomProperties.Clear();
				PopulatePropertyGrid(editPropertyGrid, editTable, false);
				bCommit.Enabled = false;
				return;
			}
			UpdateGrid();
			UpdateGrid();
		}

		private void udObs_ValueChanged(object sender, System.EventArgs e)
		{
			if (!bSupressUpdates)
				UpdateGrid();
		}

		private void bInsert_Click(object sender, System.EventArgs e)
		{
			if (!ValidateTableContents(insertTable))
			{
				MessageBox.Show("Please make sure that all the fields have been filled in with appropriate values before inserting a row into the data source.");
				return;
			}
			
			bSupressUpdates = true;
			String strCode = BuildInsertSqlCode();
			if (!bEmptyDataSource)  //only increment the maximum if we are adding into a non empty data source, otherwise, we need to prime the pump as it were
				udObs.Maximum++;
			else
				bEmptyDataSource = false;
			UpdateData(strCode);
			editTable.CustomProperties.Clear();
			CopyTable(insertTable, editTable);
			editPropertyGrid.SelectedObject = editTable;
			insertTable.CustomProperties.Clear();
			PopulatePropertyGrid(insertPropertyGrid, insertTable, false);
			udObs.Value = udObs.Maximum;
			bCommit.Enabled = true;
			bSupressUpdates = false;
		}

		#endregion
	}
}
