using System;
using System.Text;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter

// SAS AddIns namespace
using SAS.Shared.AddIns;

namespace SAS.CustomTasks.LoanCompare
{
	/// <summary>
	/// A custom task for use in SAS Enterprise Guide or the SAS Add-In for Microsoft Office
	/// </summary>
	public class LoanCompare : SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask
	{
		// change these values for your task
		#region private members for boilerplate values
		
		// replace these boilerplate values with values that apply to your task
		private string sLabel = "Custom Task";
		private string sAddInName = "Loan Compare";
		private string sAddInDescription = "Compares Loans";
		private string sProductsReq = "BASE";
		private string sProductsOpt = "";
		private string sTaskName= "Loan Compare";
		private string sCategory = "SAS Custom Task";
		private string sTaskDescription = "Compares Loans";
		private string sFriendlyName = "Loan Compare";
		private string sWhatIsDescription = "Compares Loans";
		
		#endregion
		
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;

		public LoanCompare()
		{
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;
	
			// this is a good time to get the consumer.ActiveData, if your task requires it.

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				// Does this task generate ODS-style output?
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				// PLACEHOLDER: replace with your own category
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID generated by the template
				return "EB986C34-3EBE-4629-AA26-7E48E4AC972C";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				// Does your task require input data from the application?
				return false;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.LoanCompare.LoanCompare.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				// Return true if your task generates SAS program code
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				// Replace with a user-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}

		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// Cleanup as needed
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			// no output data created
			Source = null;
			Label = null;
			return SAS.Shared.AddIns.OutputData.Unknown;

		}

		public string SasCode
		{
			get
			{	
				StringBuilder	sb = new StringBuilder();
				string			sCode;

				sb.AppendFormat( "proc loan{0}{1}{2};\r\n",
					bBasicInfo ? "" : " NOSUMMARYPRINT",
					bAmortTable ? " SCHEDULE" : "",
					bPayGraph ? " OUTSUM=work.pfj" : ""
					);

				if ( sLoan1Code.Length > 0 )
					sb.Append( sLoan1Code );

				if ( sLoan2Code.Length > 0 )
					sb.Append( sLoan2Code );

				if ( sLoan3Code.Length > 0 )
					sb.Append( sLoan3Code );

				if ( sLoan4Code.Length > 0 )
					sb.Append( sLoan4Code );

				if ( bCompare )
					sb.Append( "\tcompare;\r\n" );
				else
					sb.Append( "\tcompare NOCOMPRINT;\r\n" );

				sb.Append( "\r\n" );

				if ( bPayGraph )
				{
					string codetemplate = Global.ReadFileFromAssembly("SAS.CustomTasks.LoanCompare.LoanCompare.sas");
					sCode = sb.ToString() + codetemplate;
				}
				else
					sCode = sb.ToString();

				return sCode;
			}
		}

		public bool Initialize()
		{
			// initialize this instance of your task
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// Show the default form for this custom task
			LoanCompareForm dlg = new LoanCompareForm();
			dlg.Text = sLabel;
			dlg.taskModel = this;

			if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
				return SAS.Shared.AddIns.ShowResult.RunNow;
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}

		public int OutputDataCount
		{
			get
			{				
				// does this task create any output data sets?
				return 0;
			}
		}

		#endregion

		// Augment these methods to save and retrieve the state of your task
		#region Helper methods for serialization 

		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("LoanCompare");

			// add more WriteElement calls here to save the state of your task
			writer.WriteElementString("BasicInfoFlag",	bBasicInfo.ToString());
			writer.WriteElementString("AmortTableFlag",	bAmortTable.ToString());
			writer.WriteElementString("CompareFlag",	bCompare.ToString());
			writer.WriteElementString("PayGraphFlag",	bPayGraph.ToString());
			writer.WriteElementString("NumLoans",		iNumLoans.ToString());

			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("LoanCompare");

					// add more ReadElement calls here to read in the state of your task
					bBasicInfo =	reader.ReadElementString( "BasicInfoFlag" ) == "True" ? true : false;
					bAmortTable =	reader.ReadElementString( "AmortTableFlag" ) == "True" ? true : false;
					bCompare =		reader.ReadElementString( "CompareFlag" ) == "True" ? true : false;
					bPayGraph =		reader.ReadElementString( "PayGraphFlag" ) == "True" ? true : false;
					iNumLoans = Convert.ToInt32(reader.ReadElementString( "NumLoans" ));

					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}

		#endregion

		#region "Task state and properties"

		// generate compare statement
		private bool _bBasicInfo = true;
		public bool bBasicInfo
		{
			set { _bBasicInfo = value; }
			get { return _bBasicInfo; }
		}

		private bool _bAmortTable = false;
		public bool bAmortTable
		{
			set { _bAmortTable = value; }
			get { return _bAmortTable; }
		}

		private bool _bCompare = true;
		public bool bCompare
		{
			set { _bCompare = value; }
			get { return _bCompare; }
		}

		private bool _bPayGraph = false;
		public bool bPayGraph
		{
			set { _bPayGraph = value; }
			get { return _bPayGraph; }
		}

		// need to add # of loans
		private int	_iNumLoans = 0;
		public int iNumLoans
		{
			set { _iNumLoans = value; }
			get { return _iNumLoans; }
		}

		// SAS code for Loan 1
		private string _sLoan1Code = "";
		public string sLoan1Code
		{
			set	{ _sLoan1Code = value; }
			get { return _sLoan1Code; }
		}

		// SAS code for Loan 2
		private string _sLoan2Code = "";
		public string sLoan2Code
		{
			set	{ _sLoan2Code = value; }
			get { return _sLoan2Code; }
		}

		// SAS code for Loan 3
		private string _sLoan3Code = "";
		public string sLoan3Code
		{
			set	{ _sLoan3Code = value; }
			get { return _sLoan3Code; }
		}

		// SAS code for Loan 4
		private string _sLoan4Code = "";
		public string sLoan4Code
		{
			set	{ _sLoan4Code = value; }
			get { return _sLoan4Code; }
		}

		// Loan 1 starting date
		private DateTime _dtpStartL1 = DateTime.Now;
		public DateTime dtpStartL1
		{
			set	{ _dtpStartL1 = value; }
			get { return _dtpStartL1; }
		}

		// Loan 2 starting date
		private DateTime _dtpStartL2 = DateTime.Now;
		public DateTime dtpStartL2
		{
			set	{ _dtpStartL2 = value; }
			get { return _dtpStartL2; }
		}

		// Loan 3 starting date
		private DateTime _dtpStartL3 = DateTime.Now;
		public DateTime dtpStartL3
		{
			set	{ _dtpStartL3 = value; }
			get { return _dtpStartL3; }
		}

		// Loan 4 starting date
		private DateTime _dtpStartL4 = DateTime.Now;
		public DateTime dtpStartL4
		{
			set	{ _dtpStartL4 = value; }
			get { return _dtpStartL4; }
		}

		#endregion
	}
}
