//--------------------------------------------------------------------------------
// Copyright (c) 2006 by SAS Institute Inc, Cary NC 27513 USA 
// 
// MergeDataTask.cs
//
// Type:	C# Source Code
// Product: Custom Task for use in SAS Enterprise Guide or 
//          the SAS Add-In for Microsoft Office
//--------------------------------------------------------------------------------

#region // Namespaces 
using System;
using System.Xml;			// for XMLTextWriter and XMLTextReader
using System.IO;			// for StreamReader and StreamWriter
using SAS.Shared.AddIns;	// SAS AddIns namespace
using System.Text;			// for StringBuilder
using System.Collections;	// for ArrayList
#endregion // Namespaces 

namespace SAS.CustomTasks.MergeData
{
	/// <summary>
	/// Merge Data custom task for use in SAS Enterprise Guide or the SAS Add-In for Microsoft Office
	/// </summary>
	public class MergeDataTask : SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask
	{
	
		#region // Private members for boilerplate values 
		
		private string sLabel = "Merge Data";
		private string sAddInName = "Merge Data";
		private string sAddInDescription = "Merge Data";
		private string sProductsReq = "BASE";
		private string sProductsOpt = "";
		private string sTaskName= "Merge Data";
		private string sCategory = "SAS Custom Task";
		private string sTaskDescription = "Merge Data";
		private string sFriendlyName = "Merge Data";
		private string sWhatIsDescription = "The MergeData Custom Example Task uses the SAS DATA step to accomplish simple match merges.";
		
		#endregion // private members for boilerplate values 


		#region // Constructor 
		
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;
		
		public MergeDataTask()
		{
		}

		#endregion // Constructor 


		#region // ISASTaskAddIn Members  

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion // ISASTaskAddIn Members  


		#region // ISASTaskDescription Members 

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// Used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				// Does this task generate ODS-style output?
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				// Task category
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// Return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID
				return "4B87FDC1-66AB-46D4-9A44-69693C381F99";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				// Does your task require input data from the application?
				return false;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.MergeData.MergeDataTask.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				// Return true if your task generates SAS program code
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				// User-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}

		#endregion // ISASTaskDescription Members 


		#region // ISASTask Members 

		#region // ShowRunLog  
		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}
		#endregion // RunLog  


		#region // XmlState  
		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}
		#endregion // XmlState  


		#region // Terminate  
		public void Terminate()
		{
			// Cleanup as needed
		}
		#endregion // Terminate  


		#region // OutputDataInfo  
		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{

			// Add the output data to the EG project tree...
			Source = sOutputLibrary + "." + sOutputMemberName;
			Label = "Merged " + sOutputLibrary + "." + sOutputMemberName;
			return SAS.Shared.AddIns.OutputData.Libref;

		}
		#endregion // OutputDataInfo  


		#region // SasCode  
		public string SasCode
		{
			get
			{	
				
				StringBuilder code = new StringBuilder();

				if ( MergeType == SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.CommonVariable )
				{
					
					#region // Create the SAS code for the Common Variable Merge type  
															
					ISASTaskConsumer2 tc2 = consumer as ISASTaskConsumer2;
					if (tc2 != null)
					{

						#region // Define needed SAS macro variables...  

						int dsCount = tc2.InputData.Length;
						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("%let _DS{0}={1};\r\n", i+1, tc2.InputData[i].File));
						}

						code.Append(string.Format("%let _ID={0};\r\n%let _OUTLIBRARY={1};\r\n%let _OUTMEMNAME={2};\r\n", sID, sOutputLibrary, sOutputMemberName ));

						#endregion // Define needed SAS macro variables...  


						#region // Generate sort code...  
					
						code.Append("\r\n");
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	  Sort the data by the common ID variable... \r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");

						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("PROC SORT DATA=&_DS{0} \r\n", i+1));
							code.Append(string.Format("\tOUT=WORK.SORTSortedDS{0}(LABEL=\"Sorted &_DS{0}\");\r\n", i+1));
							code.Append("\tBY &_ID; \r\n");
							code.Append("RUN; \r\n");
							code.Append("\r\n");
						}
						#endregion // Generate sort code...  


						#region // Generate merge code...  
					
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	Merge the Data... \r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");
						code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; \r\n");
						code.Append("\tMERGE");

						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format(" WORK.SORTSortedDS{0}", i+1));
						}

						code.Append("; \r\n");
						code.Append("\tBY &_ID; \r\n");
						code.Append("RUN; \r\n");
						code.Append("\r\n");
						#endregion // Generate merge code...  


						#region // Show the merge code...  
					
						if ( this.bShowOutputResults )
						{
							code.Append("/* ------------------------------------------------------------------- \r\n");
							code.Append("	Show the merged data... \r\n");
							code.Append("------------------------------------------------------------------- */ \r\n");
							code.Append("\r\n");
							code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; \r\n");
							code.Append("RUN; \r\n");
						}
						#endregion // Show the merge code...  

					}
					#endregion // Create the SAS code for the Common Variable Merge type  

				}
				else if ( MergeType == SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.CommonVariableMergeOutputMatches )
				{
					
					#region // Create the SAS code for the CommonVariableMergeOutputMatches Merge type  
															
					ISASTaskConsumer2 tc2 = consumer as ISASTaskConsumer2;
					if (tc2 != null)
					{

						#region // Define needed SAS macro variables...  

						int dsCount = tc2.InputData.Length;
						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("%let _DS{0}={1};\r\n", i+1, tc2.InputData[i].File));
						}

						code.Append(string.Format("%let _ID={0};\r\n%let _OUTLIBRARY={1};\r\n%let _OUTMEMNAME={2};\r\n", sID, sOutputLibrary, sOutputMemberName ));

						#endregion // Define needed SAS macro variables...  


						#region // Generate sort code...  
					
						code.Append("\r\n");
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	  Sort the data by the common ID variable... \r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");

						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("PROC SORT DATA=&_DS{0} \r\n", i+1));
							code.Append(string.Format("\tOUT=WORK.SORTSortedDS{0}(LABEL=\"Sorted &_DS{0}\");\r\n", i+1));
							code.Append("\tBY &_ID; \r\n");
							code.Append("RUN; \r\n");
							code.Append("\r\n");
						}
						#endregion // Generate sort code...  


						#region // Generate merge code...  
					
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	Merge the Data...													\r\n");
						code.Append("	The IN= variable is a boolean flag that resets when a BY-Group		\r\n");
						code.Append("	changes. To output only matches from the MERGE, test for the		\r\n");
						code.Append("	condition that the current BY-Group is found in all data sets.		\r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");
						code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; \r\n");
						code.Append("\tMERGE");

						// code.Append(string.Format(" WORK.SORTSortedDS1(IN=INMASTER)"));

						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format(" WORK.SORTSortedDS{0} (IN=IN{0})", i+1));
						}

						code.Append("; \r\n");
						code.Append("\tBY &_ID; \r\n");

						code.Append("\tIF"); 
						for (int i=0; i < dsCount-1; i++)
						{
							code.Append(string.Format(" IN{0} and", i+1));
						}
						code.Append(string.Format(" IN{0}; \r\n", dsCount));						
						code.Append("RUN; \r\n");
						code.Append("\r\n");
						#endregion // Generate merge code...  


						#region // Show the merge code...  					
						if ( this.bShowOutputResults )
						{
							code.Append("/* ------------------------------------------------------------------- \r\n");
							code.Append("	Show the merged data... \r\n");
							code.Append("------------------------------------------------------------------- */ \r\n");
							code.Append("\r\n");
							code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; \r\n");
							code.Append("RUN; \r\n");
						}
						#endregion // Show the merge code...  

					}
					#endregion // Create the SAS code for the CommonVariableMergeOutputMatches Merge type  

				}
				else if ( MergeType == SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.MasterAndTransaction )
				{
					
					#region // Create the SAS code for the MasterAndTransaction Merge type  
															
					ISASTaskConsumer2 tc2 = consumer as ISASTaskConsumer2;
					if (tc2 != null)
					{

						#region // Define needed SAS macro variables...  

						int dsCount = tc2.InputData.Length;
						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("%let _DS{0}={1};\r\n", i+1, tc2.InputData[i].File));
						}

						code.Append(string.Format("%let _ID={0};\r\n%let _OUTLIBRARY={1};\r\n%let _OUTMEMNAME={2};\r\n", sID, sOutputLibrary, sOutputMemberName ));

						#endregion // Define needed SAS macro variables...  


						#region // Generate sort code...  
					
						code.Append("\r\n");
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	  Sort the data by the common ID variable... \r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");

						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("PROC SORT DATA=&_DS{0} \r\n", i+1));
							code.Append(string.Format("\tOUT=WORK.SORTSortedDS{0}(LABEL=\"Sorted &_DS{0}\");\r\n", i+1));
							code.Append("\tBY &_ID; \r\n");
							code.Append("RUN; \r\n");
							code.Append("\r\n");
						}
						#endregion // Generate sort code...  


						#region // Generate merge code...  
					
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	Merge the Data... \r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");
						code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; \r\n");
						code.Append("\tINMASTER=0;\r\n");
						code.Append("\tMERGE");

						code.Append(string.Format(" WORK.SORTSortedDS1(IN=INMASTER)"));

						for (int i=1; i < dsCount; i++)
						{
							code.Append(string.Format(" WORK.SORTSortedDS{0}", i+1));
						}

						code.Append("; \r\n");
						code.Append("\tBY &_ID; \r\n");
						code.Append("\tIF INMASTER; \r\n"); 
						code.Append("RUN; \r\n");
						code.Append("\r\n");
						#endregion // Generate merge code...  


						#region // Show the merge code...  
						if ( this.bShowOutputResults )
						{
							code.Append("/* ------------------------------------------------------------------- \r\n");
							code.Append("	Show the merged data... \r\n");
							code.Append("------------------------------------------------------------------- */ \r\n");
							code.Append("\r\n");
							code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; \r\n");
							code.Append("RUN; \r\n");
						}
						#endregion // Show the merge code...  

					}
					#endregion // Create the SAS code for the MasterAndTransaction Merge type  

				}
				else
				{

					#region // Create the SAS code for the Dataset Order Merge type  

					#region // Define needed SAS macro variables...  
					
					ISASTaskConsumer2 tc2 = consumer as ISASTaskConsumer2;
					if (tc2 != null)
					{
						int dsCount = tc2.InputData.Length;
						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format("%let _DS{0}={1};\r\n", i+1, tc2.InputData[i].File));	
						}

						code.Append(string.Format("%let _OUTLIBRARY={0};\r\n%let _OUTMEMNAME={1};\r\n", sOutputLibrary, sOutputMemberName ));

						#endregion // Define needed SAS macro variables...  


						#region // Generate merge code...  
					
						code.Append("/* ------------------------------------------------------------------- \r\n");
						code.Append("	Merge the Data... \r\n");
						code.Append("------------------------------------------------------------------- */ \r\n");
						code.Append("\r\n");
						code.Append("DATA &_OUTLIBRARY..&_OUTMEMNAME; \r\n");
						code.Append("\tMERGE");

						for (int i=0; i < dsCount; i++)
						{
							code.Append(string.Format(" &_DS{0}", i+1));
						}

						code.Append("; \r\n");
						code.Append("RUN; \r\n");
						code.Append("\r\n");
						#endregion // Generate merge code...  


						#region // Show the merge code...  
						if ( this.bShowOutputResults )
						{
							code.Append("/* ------------------------------------------------------------------- \r\n");
							code.Append("	Show the merged data... \r\n");
							code.Append("------------------------------------------------------------------- */ \r\n");
							code.Append("\r\n");
							code.Append("PROC PRINT DATA=&_OUTLIBRARY..&_OUTMEMNAME NOOBS; \r\n");
							code.Append("RUN; \r\n");
						}
						#endregion // Show the merge code...  
					}
					#endregion // Create the SAS code for the Dataset Order Merge type  

				}				
				
				// Return the SAS code to submit.
				return code.ToString();		

			}


		}
		#endregion // SasCode  


		#region // Initialize  
		public bool Initialize()
		{
			// Initialize the output name with a random name.
			Random r = new Random();
			sOutputMemberName = "MERGEDATA" + r.Next(1000,100000).ToString();

			// initialize this instance of your task
			return true;
		}
		#endregion // Initialize  


		#region // Show  
		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// Show the default form for this custom task
			MergeDataTaskForm dlg = new MergeDataTaskForm();
			dlg.Text = sLabel;

			// set the consumer interface so the dialog can call back into the
			// application
			dlg.Consumer = consumer;
			dlg.taskModel = this;

			if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
				return SAS.Shared.AddIns.ShowResult.RunNow;
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}
		#endregion // Show  


		#region // Label  
		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}
		#endregion // Label  


		#region // OutputDataCount  
		public int OutputDataCount
		{
			get
			{
				// does this task create any output data sets?
				return 1;
			}
		}
		#endregion // OutputDataCount  


		#endregion // ISASTask Members 

		
		#region // Task state and properties  

		private string _sID = "";
		public string sID
		{
			set	{ _sID = value; }
			get { return _sID; }
		}

		private string _sOutputServer = "LOCAL";
		public string sOutputServer
		{
			set { _sOutputServer = value; }
			get { return _sOutputServer; }
		}
		
		private string _sOutputLibrary = "WORK";
		public string sOutputLibrary
		{
			set	{ _sOutputLibrary = value; }
			get { return _sOutputLibrary; }
		}
		
		private string _sOutputMemberName = "MERGEDATA";
		public string sOutputMemberName
		{
			set	{ _sOutputMemberName = value; }
			get { return _sOutputMemberName; }
		}

		public enum eMergeType
		{
			CommonVariable,
			DatasetOrder,
			MasterAndTransaction,
			CommonVariableMergeOutputMatches
		}
		private eMergeType _eMergeType = eMergeType.CommonVariable;

		public eMergeType MergeType
		{
			get { return _eMergeType; }
			set { if (value != _eMergeType) 
				  { _eMergeType = value; } 
				}
		}

		private bool _bShowOutputResults = true;
		public bool bShowOutputResults
		{
			set { _bShowOutputResults = value; }
			get { return _bShowOutputResults; }
		}

		#endregion // Task state and properties  

		
		#region // Helper methods for serialization 


		#region // WriteXML  
		private string WriteXML()
		{

			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("MergeDataTask");
			
			// WriteElement calls to save the state of the task
			writer.WriteElementString( "ID", sID );
			writer.WriteElementString( "OutputServer", sOutputServer );
			writer.WriteElementString( "OutputLibrary", sOutputLibrary );
			writer.WriteElementString( "OutputMemberName", sOutputMemberName );
			writer.WriteElementString( "OutputResults", bShowOutputResults.ToString().ToUpper() );
			writer.WriteElementString( "MergeType", MergeType.ToString() );
			
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}
		#endregion // WriteXML  


		#region // ReadXML  
		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("MergeDataTask");
					
					// ReadElement calls to read in the state of the task

					sID = reader.ReadElementString("ID");				
					sOutputServer = reader.ReadElementString("OutputServer");
					sOutputLibrary = reader.ReadElementString("OutputLibrary");
					sOutputMemberName = reader.ReadElementString("OutputMemberName");
					string strShowOutputResults = reader.ReadElementString("OutputResults");
					if ( strShowOutputResults == "TRUE" )
						bShowOutputResults = true;
					else
						bShowOutputResults = false;


					string strMergeType = reader.ReadElementString("MergeType");
					if ( strMergeType == "CommonVariable" )
						MergeType = SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.CommonVariable;
					else if ( strMergeType == "CommonVariableMergeOutputMatches" )
						MergeType = SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.CommonVariableMergeOutputMatches;
					else if ( strMergeType == "MasterAndTransaction" )
						MergeType = SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.MasterAndTransaction;
					else
						MergeType = SAS.CustomTasks.MergeData.MergeDataTask.eMergeType.DatasetOrder;

					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}
		#endregion // ReadXML  

		#endregion // Helper methods for serialization 

	}
}
