using System;
using System.Drawing;
using System.Reflection;
using System.Resources;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter

// SAS AddIns namespace
using SAS.Shared.AddIns;

namespace SAS.CustomTasks.PairedBar
{
	/// <summary>
	/// A custom task for use in SAS Enterprise Guide or the SAS Add-In for Microsoft Office
	/// </summary>
	public class PairedBar : SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask
	{
		#region statics
		static Assembly assembly = Assembly.GetExecutingAssembly();
		static ResourceManager _ResourceManager = new ResourceManager(assembly.GetName().Name + ".Translate", assembly);				
		private static readonly log4net.ILog _logger = log4net.LogManager.GetLogger(typeof(PairedBar));
		#endregion

		// change these values for your task
		#region private members for boilerplate values
		
		// replace these boilerplate values with values that apply to your task
		private string sLabel = "Paired-Bar Chart";
		private string sAddInName = "Paired-Bar Chart";
		private string sAddInDescription = "Horizontal bar charts extending left & right respectively from a common vertical axis.";
		private string sProductsReq = "BASE GRAPH";
		private string sProductsOpt = "";
		private string sTaskName= "Paired-Bar Chart";
		private string sCategory = "SAS Custom Task";
		private string sTaskDescription = "Horizontal bar charts extending left & right respectively from a common vertical axis.";
		private string sFriendlyName = "Paired-Bar Chart";
		private string sWhatIsDescription = "Horizontal bar charts extending left & right respectively from a common vertical axis.";
		
		#endregion
		
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;

		public PairedBar()
		{
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;
	
			// this is a good time to get the consumer.ActiveData, if your task requires it.
			sDataSource = consumer.ActiveData.Library + "." + consumer.ActiveData.Member;

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				// Does this task generate ODS-style output?
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID generated by the template
				return "64BB7AAB-C0E6-4C15-9824-C1724FC5CD1F";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				// Does your task require input data from the application?
				return true;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.PairedBar.PairedBar.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 2;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				// Return true if your task generates SAS program code
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				// Replace with a user-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return sProductsReq;
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}
		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}

		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// Cleanup as needed
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			// no output data created
			Source = null;
			Label = null;
			return SAS.Shared.AddIns.OutputData.Unknown;

		}

		public string SasCode
		{
			get
			{	
				// Read the code file from an embedded SAS file
				// If there are any substitutions or parameters to set for the SAS program,
				// modify the string accordingly before returning it.
				string sLetData = string.Format("%let _MYDATA={0};\r\n", sDataSource );
				string sLetVar = string.Format("%let _MYCHARTVAR={0};\r\n%let _MGROUP1VAL={1};\r\n%let _MGROUP2VAL={2};\r\n", sChartVar, sGroup1Var, sGroup2Var );
				string sLetTitle = string.Format("%let _TITLE_TEXT=%STR({0});\r\n", sTitle );
				string sLetFoot = string.Format("%let _FOOTNOTE_TEXT=%STR({0});\r\n", sFootnote );
				string sLetColors = string.Format("%let _GROUP1_COLOR={0};\r\n%let _GROUP2_COLOR={1};\r\n", sClr1, sClr2 );
				string sLetRef = string.Format("%let _MAX_REF_VALUE={0};\r\n%let _REF_INTERVAL={1};\r\n", i64RefMax, i64RefMax / iIntervals );
				string sLet3D = string.Format("%let _BAR_TYPE={0};\r\n", b3D ? "HBAR3D" : "HBAR" );
				string sLetAll = sLetData + sLetVar + sLetTitle + sLetFoot + sLetColors + sLetRef + sLet3D;
				string codetemplate = Global.ReadFileFromAssembly("SAS.CustomTasks.PairedBar.PairedBar.sas");
				string sCode = sLetAll + codetemplate;
				return sCode;
			}
		}

		public bool Initialize()
		{
			// initialize this instance of your task
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// Show the default form for this custom task
			PairedBarForm dlg = new PairedBarForm();
			dlg.Text = sLabel;
			dlg.consumer = consumer;
			dlg.taskModel = this;

			if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
				return SAS.Shared.AddIns.ShowResult.RunNow;
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}

		public int OutputDataCount
		{
			get
			{				
				// does this task create any output data sets?
				return 0;
			}
		}

		#endregion

		// Augment these methods to save and retrieve the state of your task
		#region Helper methods for serialization 

		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("PairedBar");

			// add more WriteElement calls here to save the state of your task
			writer.WriteElementString( "ChartVar", sChartVar );
			writer.WriteElementString( "Group1Var", sGroup1Var );
			writer.WriteElementString( "Group2Var", sGroup2Var );

			writer.WriteElementString( "Group1ColorR", clrGroup1.R.ToString() );
			writer.WriteElementString( "Group1ColorG", clrGroup1.G.ToString() );
			writer.WriteElementString( "Group1ColorB", clrGroup1.B.ToString() );
			writer.WriteElementString( "Group2ColorR", clrGroup2.R.ToString() );
			writer.WriteElementString( "Group2ColorG", clrGroup2.G.ToString() );
			writer.WriteElementString( "Group2ColorB", clrGroup2.B.ToString() );

			writer.WriteElementString( "Group1SASColor", sClr1 );
			writer.WriteElementString( "Group2SASColor", sClr2 );
			writer.WriteElementString( "MaxRefLine", i64RefMax.ToString() );
			writer.WriteElementString( "RefIntervals", iIntervals.ToString() );
			writer.WriteElementString( "Title", sTitle );
			writer.WriteElementString( "Footnote", sFootnote );

			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("PairedBar");

					// add more ReadElement calls here to read in the state of your task
					sChartVar = reader.ReadElementString("ChartVar");
					sGroup1Var = reader.ReadElementString("Group1Var");
					sGroup2Var = reader.ReadElementString("Group2Var");

					int		iR, iG, iB;
					iR = Convert.ToInt32(reader.ReadElementString( "Group1ColorR" ));
					iG = Convert.ToInt32(reader.ReadElementString( "Group1ColorG" ));
					iB = Convert.ToInt32(reader.ReadElementString( "Group1ColorB" ));
					clrGroup1 = Color.FromArgb(iR,iG,iB);
					iR = Convert.ToInt32(reader.ReadElementString( "Group2ColorR" ));
					iG = Convert.ToInt32(reader.ReadElementString( "Group2ColorG" ));
					iB = Convert.ToInt32(reader.ReadElementString( "Group2ColorB" ));
					clrGroup2 = Color.FromArgb(iR,iG,iB);

					string s1 = reader.ReadElementString( "Group1Color" );
					string s2 = reader.ReadElementString( "Group2Color" );
					sClr1 = reader.ReadElementString( "Group1SASColor" );
					sClr2 = reader.ReadElementString( "Group2SASColor" );
					i64RefMax = Convert.ToInt64(reader.ReadElementString( "MaxRefLine" ));
					iIntervals = Convert.ToInt32(reader.ReadElementString( "RefIntervals" ));
					sTitle = reader.ReadElementString("Title");
					sFootnote = reader.ReadElementString("Footnote");

					reader.ReadEndElement();
					reader.Close();
				}
				catch( Exception ex )
				{
					// for now we just log this
					_logger.ErrorFormat("ReadXML() exception: {0}", ex.Message );
				}
			}
		}

		#endregion

		#region "Task state and properties"
		private string _sDataSource = "";
		public string sDataSource
		{
			set { _sDataSource = value; }
			get { return _sDataSource; }
		}
			
		private string _sChartVar = "";
		public string sChartVar
		{
			set	{ _sChartVar = value; }
			get { return _sChartVar; }
		}

		private string _sGroup1Var = "";
		public string sGroup1Var
		{
			set	{ _sGroup1Var = value; }
			get { return _sGroup1Var; }
		}

		private string _sGroup2Var = "";
		public string sGroup2Var
		{
			set	{ _sGroup2Var = value; }
			get { return _sGroup2Var; }
		}

		// SAS "Sea Green" == CX339966 == RGB(51,153,102)
		private System.Drawing.Color _clrGroup1 = System.Drawing.Color.FromArgb(51,153,102);
		public System.Drawing.Color clrGroup1
		{
			set	{ _clrGroup1 = value; }
			get { return _clrGroup1; }
		}

		// SAS "Sky Blue" == CX339966 == RGB(51,153,102)
		private System.Drawing.Color _clrGroup2 = System.Drawing.Color.FromArgb(0,204,255);
		public System.Drawing.Color clrGroup2
		{
			set	{ _clrGroup2 = value; }
			get { return _clrGroup2; }
		}

		private string _sClr1 = "CX339966";
		public string sClr1
		{
			set	{ _sClr1 = value; }
			get { return _sClr1; }
		}

		private string _sClr2 = "CX00CCFF";
		public string sClr2
		{
			set	{ _sClr2 = value; }
			get { return _sClr2; }
		}

		private Int64 _i64RefMax = 100;
		public Int64 i64RefMax
		{
			set	{ _i64RefMax = value; }
			get { return _i64RefMax; }
		}

		private int _iIntervals = 10;
		public int iIntervals
		{
			set	{ _iIntervals = value; }
			get { return _iIntervals; }
		}

		private string _sTitle = "";
		public string sTitle
		{
			set	{ _sTitle = value; }
			get { return _sTitle; }
		}

		// default footnote is similar to default EG footnote
		private string _sFootnote = "Generated by the SAS System on %SYSFUNC(DATE(), EURDFDE9.) at %SYSFUNC(TIME(), TIMEAMPM8.)";
		public string sFootnote
		{
			set	{ _sFootnote = value; }
			get { return _sFootnote; }
		}
		
		private bool _b3D;
		public bool b3D
		{
			set	{ _b3D = value; }
			get { return _b3D; }
		}
		#endregion

	}
}
