/* parameters coming from the task
%let _MYDATA= datasource
%let _MYCHARTVAR= var to chart
%let _MGROUP1VAL= variable for "group 1"
%let _MGROUP2VAL= variable for "group 2"
%let _TITLE_TEXT= "Title"
%let _FOOTNOTE_TEXT= "Footnote"
%let _GROUP1_COLOR= bar color
%let _GROUP2_COLOR= bar color
%let _MAX_REF_VALUE= maximum reference line value
%let _REF_INTERVAL= number of reference line intervals
%let _BAR_TYPE= hbar or hbar3D
*/

%let name=PairedBar;

/* You have to make some values 'negative' so they'll show up on
   the left side of the zero axis, but you want their labels to
   show the positive value.  Use a user-defined format to 
   accomplish this. */
proc format; picture posval low-high='000,009'; run;

data a;
 set &_MYDATA;
 format amount posval.;
 ITEM=&_MYCHARTVAR;
 illusion="&_MGROUP1VAL"; amount = -1 * &_MGROUP1VAL; output;
 illusion="&_MGROUP2VAL"; amount = &_MGROUP2VAL; output;
run;

data a; set a;
length htmlvar $500;
htmlvar='title='||quote(
 'Item: '|| trim(left(&_MYCHARTVAR)) ||'0D'x||
 'Group 1: '|| trim(left(&_MGROUP1VAL)) ||'0D'x||
 'Group 2: '|| trim(left(&_MGROUP2VAL))
);
run;

ODS HTML (ID=EGHTML)
style=minimal
gtitle gfootnote;

goptions noborder;
goptions gunit=pct htitle=6 ftitle="swissb" htext=5 ftext="swiss";

axis1 label=none;  
axis2 label=none order=(-&_MAX_REF_VALUE to &_MAX_REF_VALUE by &_REF_INTERVAL) minor=none offset=(0,0) value=(h=3pct);

pattern1 v=solid color=&_GROUP1_COLOR;
pattern2 v=solid color=&_GROUP2_COLOR;

legend1 label=none position=(bottom) cframe=white
 shape=bar(3,3) cborder=white across=2;

title "&_TITLE_TEXT";
footnote "&_FOOTNOTE_TEXT";

proc gchart data=a; 
 &_BAR_TYPE item / discrete 
 type=sum sumvar=amount 
 subgroup=illusion /* this controls the coloring */
 nostats
 maxis=axis1 /* midpoint axis */
 raxis=axis2 /* response/numeric axis */
 autoref /* reflines at every major axis tickmark */
 clipref /* put reflines 'behind' the bars */
 legend=legend1
 coutline=same
 html=htmlvar
 des="" name="&name" ;  
run;

quit;
 
