using System;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Text;		// for StringBuilder

// SAS AddIns namespace
using SAS.Shared.AddIns;


namespace SAS.CustomTasks.SPSS
{
	/// <summary>
	/// A custom task for use in SAS Enterprise Guide or the SAS Add-In for Microsoft Office
	/// </summary>
	public class SPSS : SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask
	{
		// change these values for your task
		#region private members for boilerplate values
		
		// replace these boilerplate values with values that apply to your task
		private string sLabel = "SPSS";
		private string sAddInName = "SPSS";
		private string sAddInDescription = "Converts SPSS files to SAS DataSets";
		private string sProductsReq = "BASE";
		private string sProductsOpt = "";
		private string sTaskName= "SPSS";
		private string sCategory = "SAS Custom Task";
		private string sTaskDescription = "Converts SPSS files to SAS DataSets";
		private string sFriendlyName = "SPSS";
		private string sWhatIsDescription = "Converts SPSS files to SAS Datasets";
		private String strSASCode;
		private String strFilePath;
		private String strFileName;
		
		#endregion
		
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;

		public SPSS()
		{
			strSASCode = String.Empty;
			strFilePath = String.Empty;
			strFileName = String.Empty;
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;
	
			// this is a good time to get the consumer.ActiveData, if your task requires it.

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				// Does this task generate ODS-style output?
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				// PLACEHOLDER: replace with your own category
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID generated by the template
				return "9DBDEFF5-09B6-4A8F-9F05-4108C3E81E77";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				// Does your task require input data from the application?
				return false;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.SPSS.SPSS.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				// Return true if your task generates SAS program code
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				// Replace with a user-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}

		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// Cleanup as needed
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			Source = string.Format("WORK.SPSSEngine" + strFileName);
			Label = strFileName;
			return SAS.Shared.AddIns.OutputData.Libref;
		}

		public string SasCode
		{
			get
			{
				return strSASCode;
			}
		}

		public bool Initialize()
		{
			// initialize this instance of your task
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// Show the default form for this custom task
			SPSSForm dlg = new SPSSForm();
			dlg.Text = sLabel;
			dlg.FilePath = strFilePath;

			if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
			{
				if (dlg.FilePath.Equals(String.Empty))
					return SAS.Shared.AddIns.ShowResult.Canceled;
				else if (dlg.FilePath.IndexOf(".por") < 0)
					return SAS.Shared.AddIns.ShowResult.Canceled;
				else
				{
					strFilePath = dlg.FilePath;
					strFileName = dlg.FileName;
					strSASCode = BuildProcSPSSCode();
					return SAS.Shared.AddIns.ShowResult.RunNow;
				}
			}
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		private String BuildProcSPSSCode()
		{
			StringBuilder sb = new StringBuilder();
			if (consumer!=null)
			{
				// build up the code based on the settings
				sb.AppendFormat("libname spssfile spss '" + strFilePath + "';\n");
				sb.AppendFormat("PROC SQL;\n\t");
				sb.AppendFormat("CREATE VIEW WORK.SPSSEngine" + strFileName +"\n\t\t");
				sb.AppendFormat("AS SELECT * FROM spssfile._all_;\n");
				sb.AppendFormat("QUIT;");
			}
			return sb.ToString();
		}

		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}

		public int OutputDataCount
		{
			get
			{				
				// does this task create any output data sets?
				return 1;
			}
		}

		#endregion

		// Augment these methods to save and retrieve the state of your task
		#region Helper methods for serialization 

		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("SPSS");
			writer.WriteElementString("FilePath", strFilePath);
			writer.WriteElementString("FileName", strFileName);
			// add more WriteElement calls here to save the state of your task
			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("SPSS");
					strFilePath = reader.ReadElementString("FilePath");
					strFileName = reader.ReadElementString("FileName");
					// add more ReadElement calls here to read in the state of your task
					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}

		#endregion
	}
}
