using System;
using System.Xml;		// for XMLTextWriter and XMLTextReader
using System.IO;		// for StreamReader and StreamWriter
using System.Windows.Forms;

// SAS AddIns namespace
using SAS.Shared.AddIns;

namespace SAS.CustomTasks.ZipcodeDotMap
{
	/// <summary>
	/// A custom task for use in SAS Enterprise Guide or the SAS Add-In for Microsoft Office
	/// </summary>
	public class ZipcodeDotMap : SAS.Shared.AddIns.ISASTaskAddIn, SAS.Shared.AddIns.ISASTaskDescription, SAS.Shared.AddIns.ISASTask
	{
		public const string	strFileINI = @"\SAS\Enterprise Guide\4\ZipcodeDotMap.ini";

		// change these values for your task
		#region private members for boilerplate values
		
		// replace these boilerplate values with values that apply to your task
		private string sLabel = "Custom Task";
		private string sAddInName = "Zipcode Dot Map";
		private string sAddInDescription = "Description of Zipcode Dot Map";
		private string sProductsReq = "BASE GRAPH";
		private string sProductsOpt = "";
		private string sTaskName= "Zipcode Dot Map";
		private string sCategory = "SAS Custom Task";
		private string sTaskDescription = "Description of Zipcode Dot Map";
		private string sFriendlyName = "Zipcode Dot Map";
		private string sWhatIsDescription = "What is Zipcode Dot Map";
		
		#endregion
		
		// store reference to the application Consumer object
		private ISASTaskConsumer consumer = null;

		public ZipcodeDotMap()
		{
			// nothing needed at this time
		}

		#region ISASTaskAddIn Members

		public bool VisibleInManager
		{
			get
			{				
				return true;
			}
		}

		public string AddInName
		{
			get
			{				
				return sAddInName;
			}
		}

		public void Disconnect()
		{
			// perform cleanup tasks
			consumer = null;
		}

		public int Languages(out string[] Items)
		{		
			// by default, we support English 
			// Add more languages as needed
			Items = new string[] {"en-US"};
			return 1;
		}

		public string AddInDescription
		{
			get
			{				
				return sAddInDescription;
			}
		}

		public bool Connect(ISASTaskConsumer Consumer)
		{
			// perform any initialization needed when the application connects
			consumer = Consumer;
	
			// this is a good time to get the consumer.ActiveData, if your task requires it.
			//sDataSource = consumer.ActiveData.File;
			sDataSource = consumer.ActiveData.Library + "." + consumer.ActiveData.Member;

			return true;
		}

		public string Language
		{
			set
			{
				// if you support multiple languages, add handler here
			}
		}

		#endregion

		#region ISASTaskDescription Members

		public string ProductsRequired
		{
			get
			{
				// What SAS products are required for this task to run?
				return sProductsReq;
			}
		}

		public bool StandardCategory
		{
			get
			{
				// used typically only by SAS-supplied tasks			
				return false;
			}
		}

		public bool GeneratesListOutput
		{
			get
			{
				// Does this task generate ODS-style output?
				return true;
			}
		}

		public string TaskName
		{
			get
			{
				return sTaskName;
			}
		}

		public string Validation
		{
			get
			{
				// Add a validation string that makes sense for your organization
				// For example, "dev", "test", or "prod"
				return "";
			}
		}

		public string TaskCategory
		{
			get
			{
				// PLACEHOLDER: replace with your own category
				return sCategory;
			}
		}

		public string IconAssembly
		{
			get
			{
				// return the full path/name of this assembly, assuming that the
				// icon is embedded within the assembly
				return System.Reflection.Assembly.GetExecutingAssembly().Location;
			}
		}

		public string Clsid
		{
			get
			{
				// ClassID GUID generated by the template
				return "8F0F5EB3-89F4-4306-818E-E4CC66D5A5F5";
			}
		}

		public SAS.Shared.AddIns.ShowType TaskType
		{
			get
			{				
				return SAS.Shared.AddIns.ShowType.Wizard;
			}
		}

		public bool RequiresData
		{
			get
			{
				// Does your task require input data from the application?
				return true;
			}
		}

		public string IconName
		{
			get
			{
				// return the name of the icon within this assembly
				// including namespace qualifiers
				return "SAS.CustomTasks.ZipcodeDotMap.ZipcodeDotMap.ico";
			}
		}

		public int MinorVersion
		{
			get
			{
				return 0;
			}
		}

		public int MajorVersion
		{
			get
			{
				return 1;
			}
		}

		public int NumericColsRequired
		{
			get
			{
				// How many numeric variables are required in input data, if any?
				return 0;
			}
		}

		public string TaskDescription
		{
			get
			{				
				return sTaskDescription;
			}
		}

		public bool GeneratesSasCode
		{
			get
			{
				// Return true if your task generates SAS program code
				return true;
			}
		}

		public string FriendlyName
		{
			get
			{
				// Replace with a user-friendly name for your task
				return sFriendlyName;
			}
		}

		public string ProcsUsed
		{
			get
			{
				// What SAS procedures are used in this task?
				return "";
			}
		}

		public string WhatIsDescription
		{
			get
			{
				// Longer description for your task
				return sWhatIsDescription;
			}
		}

		public string ProductsOptional
		{
			get
			{
				// What SAS products are optionally used by this task?
				return sProductsOpt;
			}
		}

		#endregion

		#region ISASTask Members

		public string RunLog
		{
			get
			{
				// if your task does not generate SAS code, you can supply your own
				// log text to record the work completed.
				return "";
			}
		}

		// manage the state of the task when serializing to and from the project
		// makes use of some helper methods: WriteXML and ReadXML
		public string XmlState
		{
			get
			{
				return WriteXML();
			}
			set
			{
				ReadXML(value);
			}
		}

		public void Terminate()
		{
			// Cleanup as needed
		}

		public SAS.Shared.AddIns.OutputData OutputDataInfo(int Index, out string Source, out string Label)
		{
			// no output data created
			Source = null;
			Label = null;
			return SAS.Shared.AddIns.OutputData.Unknown;

		}

		public string SasCode
		{
			get
			{	
				// Read the code file from an embedded SAS file
				// If there are any substitutions or parameters to set for the SAS program,
				// modify the string accordingly before returning it.
				string sVarSetup = string.Format("%let _MYDATA={0};\r\n%let _MYZIPVAR={1};\r\n%let _MYRESPVAL={2};\r\n%let _TITLE_TEXT=%STR({3});\r\n%let _FOOTNOTE_TEXT=%STR({4});\r\n", sDataSource, sZipVar, sRespVar, sTitle, sFootnote );
				string codetemplate = Global.ReadFileFromAssembly("SAS.CustomTasks.ZipcodeDotMap.ZipcodeDotMap.sas");
				string sCode = sVarSetup + codetemplate;
				return sCode;
			}
		}

		public bool Initialize()
		{
			// initialize this instance of your task
			return true;
		}

		public SAS.Shared.AddIns.ShowResult Show(System.Windows.Forms.IWin32Window Owner)
		{
			// Show the default form for this custom task
			ZipcodeDotMapForm dlg = new ZipcodeDotMapForm();
			dlg.Text = sLabel;
			dlg.consumer = consumer;
			dlg.taskModel = this;

			if (dlg.ShowDialog()==System.Windows.Forms.DialogResult.OK)
				return SAS.Shared.AddIns.ShowResult.RunNow;
			else
				return SAS.Shared.AddIns.ShowResult.Canceled;
		}

		// The label this task is known by within a project
		public string Label
		{
			get
			{			
				return sLabel;
			}
			set
			{
				sLabel = value;
			}
		}

		public int OutputDataCount
		{
			get
			{				
				// does this task create any output data sets?
				return 0;
			}
		}

		#endregion

		// Augment these methods to save and retrieve the state of your task
		#region Helper methods for serialization 

		private string WriteXML()
		{
			StringWriter sw = new StringWriter();

			XmlTextWriter writer = new XmlTextWriter(sw);
			writer.WriteStartElement("ZipcodeDotMap");

			// add more WriteElement calls here to save the state of your task
			writer.WriteElementString( "ZipVar", sZipVar );
			writer.WriteElementString( "RespVar", sRespVar );
			writer.WriteElementString( "Title", sTitle );
			writer.WriteElementString( "Footnote", sFootnote );

			writer.WriteEndElement();
			writer.Close();

			return sw.ToString();
		}

		private void ReadXML(string xml)
		{
			if (xml!=null && xml.Length>0)
			{
				try
				{
					StringReader sr = new StringReader(xml);
					XmlTextReader reader = new XmlTextReader(sr);
					reader.ReadStartElement("ZipcodeDotMap");

					// add more ReadElement calls here to read in the state of your task
					sZipVar = reader.ReadElementString("ZipVar");
					sRespVar = reader.ReadElementString("RespVar");
					sTitle = reader.ReadElementString("Title");
					sFootnote = reader.ReadElementString("Footnote");

					reader.ReadEndElement();
					reader.Close();
				}
				catch
				{
				}
			}
		}

		#endregion

		#region "Task state and properties"
		private string _sDataSource = "";
		public string sDataSource
		{
			set { _sDataSource = value; }
			get { return _sDataSource; }
		}
			
		// variable containing the zipcode values
		private string _sZipVar = "";
		public string sZipVar
		{
			set	{ _sZipVar = value; }
			get { return _sZipVar; }
		}

		// variable containing the response values
		private string _sRespVar = "";
		public string sRespVar
		{
			set	{ _sRespVar = value; }
			get { return _sRespVar; }
		}

		private string _sTitle = "";
		public string sTitle
		{
			set	{ _sTitle = value; }
			get { return _sTitle; }
		}

		// default footnote is similar to default EG footnote
		private string _sFootnote = "Generated by the SAS System on %SYSFUNC(DATE(), EURDFDE9.) at %SYSFUNC(TIME(), TIMEAMPM8.)";
		public string sFootnote
		{
			set	{ _sFootnote = value; }
			get { return _sFootnote; }
		}
		#endregion
	}
}
