%macro ZIPCOD01();

/* override EG default */
GOPTIONS DEVICE=gif;

ODS HTML (ID=EGHTML)
style=minimal
gtitle gfootnote;

goptions 
gunit=pct 
htitle=6 htext=4
ftitle="arial" ftext="arial" 
ctitle='white' ctext='white';

goptions border cback=cx3366cc;

%* --- Check for required parameters --- ;

%if %length(&_MYDATA) eq 0 %then %do;
 data a; message='_MYDATA is a required parameter'; run;
 proc print data=a noobs; run;
%end;
%if %length(&_MYZIPVAR) eq 0 %then %do;
 data a; message='_MYZIPVAR is a required parameter'; run;
 proc print data=a noobs; run;
%end;
%if %length(&_MYRESPVAL) eq 0 %then %do;
 data a; message='_MYRESPVAL is a required parameter'; run;
 proc print data=a noobs; run;
%end;
%if %length(&_TITLE_TEXT) eq 0 %then %do;
 data a; message='_TITLE_TEXT is a required parameter'; run;
 proc print data=a noobs; run;
%end;

%* Make your own copy of the data to guarantee that zipcode & response values are numeric;
data foodata (keep = foozipnum fooresp st);
 set &_MYDATA;
 foozipnum=0; foozipnum=&_MYZIPVAR;
 fooresp=0; fooresp=&_MYRESPVAL;
 st='  '; st=zipstate(trim(left(foozipnum)));
run;


%* --- Assign default values to optional parameters, if needed --- ;

%* If user did not set a max scaling/size variable, use max response value in data;
%if &_MAX_VAL eq %then %do;
 proc sql noprint;
  select max(fooresp) into :_MAX_VAL from foodata;
 quit; run;
%end;

%if &_MIN_DOT eq %then 
 %let _MIN_DOT=.5;

%if &_MAX_DOT eq %then 
 %let _MAX_DOT=5;

%if %length(&_DOT_COLOR) eq 0 %then 
 %let _DOT_COLOR=cxff6666;

%if %length(&_MAP_COLOR) eq 0 %then 
 %let _MAP_COLOR=tan;

%if %length(&_DOT_OUTLINE) eq 0 %then 
 %let _DOT_OUTLINE=gray44;

%if %length(&_MAP_OUTLINE) eq 0 %then 
 %let _MAP_OUTLINE=gray;

%if %length(&_RED_GRAD) eq 0 %then 
 %let _RED_GRAD=05;

%if %length(&_GREEN_GRAD) eq 0 %then 
 %let _GREEN_GRAD=70;

%if %length(&_BLUE_GRAD) eq 0 %then 
 %let _BLUE_GRAD=B0;

%if %length(&_SHADOW_COLOR) eq 0 %then 
 %let _SHADOW_COLOR=black;



proc sql;
%* Find the x/y coordinates of the city in the zipcode; 
 create table zipcod_data as
 select unique foodata.*,
        zipcode.city, 
        zipcode.zip, 
        -1*zipcode.x as longitude, 
        zipcode.y as latitude
 from foodata left join sashelp.zipcode
 on foodata.foozipnum=zipcode.zip;
%* See if there are any zipcodes that were not found;
 create table notfound as 
 select * from zipcod_data
 where longitude=.;
quit; run;

%* Convert long/lat degrees to radians, getting ready for gproject;
data zipcod_data; set zipcod_data (where=(longitude ^=.));
  state=stfips(st);
  x=atan(1)/45 * longitude;
  y=atan(1)/45 * latitude;
run;


%* Sort the data so the larger dots are drawn first/behind the smaller ones;
proc sort data=zipcod_data out=zipcod_data;
by descending fooresp;
run;


data zipcod_data;
   length function style color $ 8 position $ 1 text $ 20 html $1024;
   retain xsys ysys '2' hsys '3' when 'a';
  set zipcod_data;
  anno_flag=1;
html= 'title='|| quote( trim(left(city))||', '||trim(left(st))||'  '||trim(left(foozipnum))||'     Value='||trim(left(fooresp))||' ');
run;


%* reduce the size of maps.states;
proc sql;
create table states_map as select * from maps.states
where fipstate(state) not in ('AK' 'HI' 'PR') and density < 4;
quit; run;
%* Strange ... when I gproject it myself, I can use 'density < 5' ... ;

%* project the map & annotate data;
data combined; set states_map zipcod_data; run;
proc gproject data=combined out=combined dupok;
  id state;
run;
data states_map zipcod_data;
  set combined;
  if anno_flag=1 then output zipcod_data;
  else output states_map;
run;


%* Annotate a dot at each zipcode;
data zipcod_data; set zipcod_data;
function='pie';
style='psolid';
color="&_DOT_COLOR";
rotate=360;
%* Calculate the size (ie, radius) of the dot;
%* To be a little more accurate, you might want to map the data value to the;
%* area of the dot, and then calculate the radius for that area;
size=&_MIN_DOT+((fooresp/&_MAX_VAL)*&_MAX_DOT);
output;

%* Draw an outline around each dot;
color="&_DOT_OUTLINE";
style='pempty';
output;
run;



%* Annotate a shadow effect around the edge of the map;
proc gremove data=states_map out=anno_shadow;
 by state; id state;
run;
data anno_shadow;
   length COLOR FUNCTION $ 8;
   retain FX FY FUNCTION;
   set anno_shadow; by state segment;
   xsys='2';
   ysys='2';
   color='black';
   size=1.75;
   when='B';
   style='msolid';
   if first.segment then do;
      FUNCTION = 'poly'; FX = X; FY = Y;
      end;
   else if FUNCTION ^= ' ' then do;
      if X = .  then do;
         X = FX; Y = FY; output; FUNCTION = ' ';
      end;
      else FUNCTION = 'polycont';
   end;
   if FUNCTION ^= ' ' then do;
      output;
      if last.segment then do;
         X = FX; Y = FY; output;
      end;
   end;
run;


%* Give a little x & y offset, so it will look like a shadow;
data anno_shadow; 
set anno_shadow; by function notsorted;
if upcase(function)='POLY' then do; 
 if  not last.function  then delete; 
end;   
 x=x+.002; y=y-.003; 
run;


%* Annotate a gradient background;
data anno_gradient;
length function $8 style $10 color $8;
retain xsys '3' ysys '3'  when 'b' style 'solid';
orig_red=  input("&_RED_GRAD",hex2.);
orig_green=input('70',hex2.);
orig_blue= input('B0',hex2.);
do i=0 to 98 by 2;
 count+1;
 function='move'; x=0; y=i; output;
 function='bar'; x=100;  y=i+2;
 percent=(count-1)/(45);
 red=orig_red     + ( (256-orig_red)   * percent );
 if red > 255 then red=255;
 green=orig_green + ( (256-orig_green) * percent );
 if green > 255 then green=255;
 blue=orig_blue   + ( (256-orig_blue)  * percent );
 if blue > 255 then blue=255;
 rgb=put(red,hex2.)||put(green,hex2.)||put(blue,hex2.);
 color="cx"||rgb;
 output;
end;
run;

%* Combine gradient and shadow into single annotate data set;
data anno_back;
 set anno_gradient anno_shadow;
 if function ne '' then output;
run;

%* Use "arial" system font on windows (it has nice smooth edges);
 %if ( &SYSSCP eq WIN ) %then %do;
  goptions gunit=pct htitle=5 htext=4 ftitle="arial/bold" ftext="arial";
 %end;
%* else, use sas/graph software fonts everywhere else;
 %else %do;
  goptions gunit=pct htitle=5 htext=4 ftitle=swiss ftext=swiss;
 %end;

 pattern v=s c=&_MAP_COLOR;

 goptions xpixels=800 ypixels=600;

 title "&_TITLE_TEXT";
%if %length(&_FOOTNOTE_TEXT) !eq 0 %then %do;
 footnote "&_FOOTNOTE_TEXT";
%end;
 proc gmap data=states_map map=states_map anno=zipcod_data;
 id state; choro state / 
  levels=1
  coutline=&_MAP_OUTLINE
  nolegend
  anno=anno_back
  des="";
 run;

title color=black "The following zipcodes were not found in sashelp.zipcode";
proc print data=notfound noobs; 
var foozipnum fooresp st;
run;

%* ==================================================================== ;

%* These individual state maps might be better calculated using another macro;
%* Rather than using this call execute trick;

title "&_TITLE_TEXT";
%if %length(&_FOOTNOTE_TEXT) !eq 0 %then %do;
 footnote "&_FOOTNOTE_TEXT";
%end;
goptions xpixels=500 ypixels=375;
%* I could duplicate the above code, and generate and gproject a new map;
%* using the county outline maps, but to save time I am just re-using the;
%* us/state map.;
proc sql noprint;
 create table uniqstates as
 select unique st from zipcod_data
 where st ne '' and longitude ne .;
quit; run;
%* Loop through, and do a map for each state;
data _null_;
 set uniqstates;
   call execute('proc sql; ');
   call execute('create table zipcod1_data as select * from zipcod_data where longitude^=. and st="'||trim(left(st))||'";' );
   call execute('create table states1_map as select * from states_map where fipstate(state) = "'||trim(left(st))||'";' );
   call execute('quit; run; ');
   call execute('title2 "State = '||trim(left(st))||'";');
   call execute('proc gmap data=states1_map map=states1_map anno=zipcod1_data;');
   call execute('id state; choro state / ');
%* I use shadow color for outline of these state maps;
   call execute(" coutline=&_SHADOW_COLOR");
   call execute(' anno=anno_gradient levels=1 nolegend des="";');
   call execute('run;');
run;
/*
*/

quit;

%mend ZIPCOD01;

options source mprint;

* Here are the macro variables I use in this stp example...;

%global _MYDATA _MYZIPVAR _MYRESPVAL _TILE_TEXT _FOOTNOTE_TEXT;
* ;
* Data and variables (required)... 
* ---------------------------------------------------------------------;
* _MYDATA = Name of your data set;
* _MYZIPVAR = Variable containing 5-digit zipcode (should be numeric);
* _MYRESPVAL = Name of variable containing your numeric response values;
* _TITLE_TEXT = Text to be used as the title;
* _FOOTNOTE_TEXT = Text to be used as the footnote;
* ---------------------------------------------------------------------;
* %let _MYDATA=/u/realliso/public_html/proj/stp/somedata.sas7bdat;
* %let _MYZIPVAR=zipnum;
* %let _MYRESPVAL=somevalue;
* %let _TITLE_TEXT=Values of 'something' by Zipcode;
* %let _FOOTNOTE_TEXT=;

%global _MAX_VAL _MIN_DOT _MAX_DOT _DOT_COLOR _MAP_COLOR _DOT_OUTLINE 
  _MAP_OUTLINE _RED_GRAD _GREEN_GRAD _BLUE_GRAD _SHADOW_COLOR;
* ;
* Size, Color, and appearance variables (optional)...;
* ---------------------------------------------------------------------;
* _MAX_VAL = Maximum value to scale all the dots to;
* _MIN_DOT = Minimum size (radius) of dots;
* _MAX_DOT = Maximum size (radius) of dots - units are % of screen;
* _DOT_COLOR = Color of the dots;
* _MAP_COLOR = Color of the states in the US map;
* _DOT_OUTLINE = Color of borders around the dots;
* _MAP_OUTLINE = Color of borders around the states;
* _RED_GRAD = Red part of hex color for gradient background;
* _GREEN_GRAD = Green part of hex color for gradient background;
* _BLUE_GRAD = Blue part of hex color for gradient background;
* _SHADOW_COLOR = Color of annotated shadow;
* ---------------------------------------------------------------------;
* %let _MAX_VAL=25;
* %let _MIN_DOT=.5;
* %let _MAX_DOT=5;
* %let _DOT_COLOR=cxff6666;
* %let _MAP_COLOR=tan;
* %let _DOT_OUTLINE=gray44;
* %let _MAP_OUTLINE=gray;
* %let _SHADOW_COLOR=black;

* Run the macro;
%ZIPCOD01;



